/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <qpushbutton.h>
#include <qlabel.h>
#include <qfont.h>
#include <math.h>

#include "calculatorimpl.h"

CalculatorImpl::CalculatorImpl( QWidget * parent, const char * name,
				WFlags f )
    : Calculator( parent, name, f )
{
    memMark = new QLabel( "m", lcd );
    memMark->setFont( QFont( "helvetica", 12, QFont::Bold, TRUE ) );
    memMark->resize( 12, 12 );
    memMark->move( 4, 2 );
    memMark->hide();
    mem = 0;
    clear();
    btnMemRecall->setEnabled( FALSE );
}


void CalculatorImpl::execOp( Operations i )
{
    if( i > oMult ){ // % 1/X X^2 
 	if( state == sNewNumber ){
	    num = acc;
 	    acc = evalExpr( i );
 	    lcd->display( acc );
 	} else {
	    state = sNewNumber;
	    num = evalExpr( i );
	    lcd->display( num );
	    acc = num;
	    num = 0;
	}
    } else if( i > oSum ){ // + - * /
	if( state == sContinue ){
	    num = evalExpr( lastOp );
	    acc = 0;
	    lcd->display( num );
	} if( state == sNewNumber ){
	    num = acc;
	} else {
	    acc = num;
	}
	lastOp = i;
    } else if( i == oChSign ){ // +/-
	if( state == sNewNumber ){
	    acc = -acc;
	    lcd->display( acc );
	} else {
	    state = sNewNumber;
	    num = -num;
	    lcd->display( num );
	    acc = num;
	    num = 0;
	}
	numDecimals = 0;
	flPoint = FALSE;
	return;
    } else if( i == oPoint ){ // .
 	flPoint = TRUE;	
 	return;
    } else if( i == oSum ){ // =
	acc = evalExpr( lastOp );
	lcd->display( acc );
    }
    lastAc = i;
    state  = sNewNumber;
    numDecimals = 0;
    flPoint = FALSE;
}

double CalculatorImpl::evalExpr( Operations op )
{
    double sum = 0;

    switch( op ){
        case oPercent: sum = num / 100; break;
        case oDivX: (num == 0) ? sum = 0 : sum = 1 / num; break;
        case oXsquared: sum = num * num; break;
        case oChSign: (state == sStart) ? sum = -num : sum = -acc; break;
	case oSub:  sum = acc - num; break;
	case oMult: sum = acc * num; break;
	case oAdd:  sum = acc + num; break;
        case oDiv:  (num == 0) ? sum = 0 : sum = acc / num; break;
	default: sum = num; break;
    }
    return sum;
}

void CalculatorImpl::enterNumber( int n )
{    
    if( state == sStart || state == sContinue ){
	if( lcd->value() > 0 ){
	    QString s = QString::number( lcd->value(), 'g', lcd->numDigits());
	    if( s.length() > (uint)(lcd->numDigits() - 2)) return;
	    
	} else if( numDecimals >= (lcd->numDigits() - 2) ){
	    return;
	}
    }

    if( state == sNewNumber ){
	state = sContinue;
	if( lastAc >= oAdd ){
	    acc = num;
	} else {
	    state = sStart;
	    acc = 0;
	}
	if( flPoint ){
	    numDecimals = 1;
	    num = n / pow(10, numDecimals);
	} else
	    num = n;
    } else if( flPoint ){
	numDecimals++;
	if( num < 0 ){
	    num -= n / pow(10, numDecimals);
	} else {
	    num += n / pow(10, numDecimals);
	}
    } else {
	num *= 10;
	if( num  < 0 )
	    num -= n;
	else
	    num += n;
    }

    // We need feedback in the calc display while entering fl.point zeros.
    // This is a small hack to display sequences like: 0.000 and 1.100
    double integer, fraction;
    fraction = modf( num, &integer );
    if( flPoint && ((numDecimals >= lcd->numDigits()) || (n == 0)) ){
	QString fake, is, fs, zeros;

	is = QString::number( integer );
	fs = QString::number( fraction );
	if( fs.contains('e') ){
	    fs = QString::number( fraction, 'f', lcd->numDigits() );
	}
	fs = fs.mid( 2, lcd->numDigits() );

	if( (integer == 0) && (fraction == 0) )
	    fake = "0.";
	else if( (integer != 0) && (fraction == 0) )
	    fake = is + ".";
	else
	    fake = is + "." + fs;

	zeros.fill( '0', (numDecimals - fs.length()) );
	fake += zeros;
	lcd->display( fake );
    } else
	lcd->display( num );
}

void CalculatorImpl::memClear()
{ 
    mem = 0; 
    memMark->hide();
    btnMemRecall->setEnabled( FALSE );
}

void CalculatorImpl::memRecall()
{
    if( state == sNewNumber ){
	acc = mem;
    } else {
	acc   = mem;
	num   = 0;
	state = sNewNumber;
    }
    lcd->display( mem );
}

void CalculatorImpl::memAdd()
{    
    if( state == sNewNumber ){
	mem += acc;
    } else {
	mem += num;
    }
    if( mem != 0 ){
	memMark->show();
	btnMemRecall->setEnabled( TRUE );
    }
}
