/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#ifndef _QIMPENINPUT_H_
#define _QIMPENINPUT_H_

#include <qframe.h>
#include <qlist.h>
#include "qimpenchar.h"
#include "qdawg.h"

class QPushButton;
class QIMPenWidget;
class QIMPenSetup;
class QIMPenWordPick;

class QIMPenInput : public QFrame
{
    Q_OBJECT
public:
    QIMPenInput( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
    virtual ~QIMPenInput();

    void addCharSet( const QString &fn );

    QSize sizeHint() const;
    
signals:
    void key( unsigned int unicode );

protected slots:
    void wordPicked( const QString & );
    void selectCharSet( QIMPenCharSet * );
    void beginStroke();
    void strokeEntered( QIMPenStroke *st );
    void processMatches( const QIMPenCharMatchList &ml );
    void setup();

protected:
    void updateWordMatch( const QIMPenCharMatchList &ml );
    void matchWords();
    void scanDict( const QDawg::Node* n, int ipos, const QString& str, int error );
    void keypress( unsigned int ch );

    struct MatchWord {
	MatchWord( const QString &w, int e ) { word = w; error = e; }
	QString word;
	int error;
    };

    class MatchWordList : public QList<MatchWord>
    {
    public:
	int compareItems( QCollection::Item item1, QCollection::Item item2 ) {
	    MatchWord *m1 = (MatchWord *)item1;
	    MatchWord *m2 = (MatchWord *)item2;
	    return m1->error - m2->error;
	}
    };

    QRect prefRect;
    QIMPenWidget *pw;
    QPushButton *setupBtn;
    QIMPenSetup *setupDlg;
    QList<QIMPenStroke> strokes;
    QIMPenChar *matchChar;
    QIMPenChar testChar;
    QIMPenCharSet *currCharSet;
    QList<QIMPenCharSet> charSets;
    unsigned lastKey;
    QList<QIMPenCharMatchList> wordChars;
    MatchWordList wordMatches;
    QString word;
    int errorThreshold;
    int goodMatches;
    int badMatches;
    int maxGuess;
    QIMPenWordPick *wordPicker;
};

#endif // _QIMPENINPUT_H_
