/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include <qwidget.h>
#include <qlist.h>
#include "qimpenchar.h"

class QIMPenWidget : public QWidget
{
    Q_OBJECT
public:
    QIMPenWidget( QWidget *parent );

    void clear();
    void removeStroke( bool greyed );

    void addCharSet( QIMPenCharSet *cs );
    void removeCharSet( QIMPenCharSet *cs );
    void showCharacter( QIMPenChar *, int speed = 10 );
    virtual QSize sizeHint();


signals:
    void changeCharSet( QIMPenCharSet *cs );
    void beginStroke();
    void stroke( QIMPenStroke *ch );

protected slots:
    void timeout();

protected:
    enum Mode { Waiting, Input, Output };
    bool selectSet( QPoint );
    virtual void mousePressEvent( QMouseEvent *e );
    virtual void mouseReleaseEvent( QMouseEvent *e );
    virtual void mouseMoveEvent( QMouseEvent *e );
    virtual void paintEvent( QPaintEvent *e );

protected:
    Mode mode;
    bool autoHide;
    QPoint lastPoint;
    unsigned pointIndex;
    int strokeIndex;
    QTimer *timer;
    QColor strokeColor;
    QRect dirtyRect;
    QIMPenChar *outputChar;
    QIMPenStroke *outputStroke;
    QIMPenStroke *inputStroke;
    QIMPenCharSet *currCharSet;
    QList<QIMPenStroke> strokes;
    QList<QIMPenCharSet> charSets;
};

