/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "global.h"
#include "qdawg.h"
#include <qfile.h>

#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

static bool createdCategories = FALSE;
static QDawg* fixed_dawg = 0;
static QDawg* added_dawg = 0;

static QString dictDir()
{
    QString dir = getenv("QPEDIR");
    if ( dir.isEmpty() ) dir = "..";
    return dir + "/etc/dict";
}

static QString addedDawgName()
{
    return dictDir() + "/local.dawg";
}

Global::Global()
{
}

const QDawg& Global::fixedDawg()
{
    if ( !fixed_dawg ) {
	if ( !createdCategories )
	    createCategories();

	fixed_dawg = new QDawg;
	QString dawgfilename = dictDir() + "/udw.dawg";
	QFile dawgfile(dawgfilename);

	if ( !dawgfile.exists() ) {
	    QString fn = dictDir() + "/words";
	    QFile in(fn);
	    in.open(IO_ReadOnly);
	    fixed_dawg->createFromWords(&in);
	    dawgfile.open(IO_WriteOnly);
	    fixed_dawg->write(&dawgfile);
	    dawgfile.close();
	} else {
	    fixed_dawg->readFile(dawgfilename);
	}
    }

    return *fixed_dawg;
}

const QDawg& Global::addedDawg()
{
    if ( !added_dawg ) {
	if ( !createdCategories )
	    createCategories();

	added_dawg = new QDawg;
	QString dawgfilename = addedDawgName();
	QFile dawgfile(dawgfilename);

	if ( dawgfile.open(IO_ReadOnly) ) {
	    added_dawg->readFile(dawgfilename);
	} else {
	    // hack so we don't have to deal with null dawgs
	    added_dawg->createFromWords(QStringList("a"));
	}
    }

    return *added_dawg;
}

void Global::addWords(const QStringList& wordlist)
{
    addedDawg();
    QStringList all = added_dawg->allWords() + wordlist;
    added_dawg->createFromWords(all);

    QString dawgfilename = addedDawgName();
    QFile dawgfile(dawgfilename);
    if ( dawgfile.open(IO_WriteOnly) ) {
	added_dawg->write(&dawgfile);
	dawgfile.close();
    }

    // #### Re-read the dawg here if we use mmap().

    // #### Signal other processes to re-read.
}

QStringList Global::categories()
{
    if ( !createdCategories )
	createCategories();
    QStringList lst;
    lst << "Business" << "Private" << "Unified" << "Important";
    return lst;
}

void Global::createCategories()
{
    createdCategories = TRUE;
    QStringList lst = categories();
    for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
	QString dir = QString( getenv("HOME") ) + "/Documents/" + *it;
	mkdir( dir.latin1(), 0755 );
    }
}
