/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "imageedit.h"
#include <qpainter.h>

ImageEdit::ImageEdit( QWidget *parent = 0, const char *name = 0 )
    : QScrollView( parent, name, WNorthWestGravity | WResizeNoErase ), buffer()
{
    buffer.resize( size() );
    buffer.fill( colorGroup().color( QColorGroup::Base ) );
}

void ImageEdit::contentsMousePressEvent( QMouseEvent *e )
{
    lastPos = e->pos();
}

void ImageEdit::contentsMouseMoveEvent( QMouseEvent *e )
{
    QPainter pw( viewport() );
    QPainter pb( &buffer );
    pb.drawLine( lastPos, e->pos() );
    pw.drawLine( contentsToViewport( lastPos ),
		 contentsToViewport( e->pos() ) );
    lastPos = e->pos();
}

void ImageEdit::contentsMouseReleaseEvent( QMouseEvent * )
{
}

void ImageEdit::viewportResizeEvent( QResizeEvent *e )
{
    buffer.resize( QMAX( e->oldSize().width(), e->size().width() ), QMAX( e->oldSize().height(), e->size().height() ) );
    QPainter p( &buffer );
    if ( e->size().width() > e->oldSize().width() )
	p.fillRect( e->oldSize().width(), 0, e->size().width() - e->oldSize().width(), e->size().height(), colorGroup().color( QColorGroup::Base ) );
    if ( e->size().height() > e->oldSize().height() )
	p.fillRect( 0, e->oldSize().height(), e->size().width(), e->size().height() - e->oldSize().height(), colorGroup().color( QColorGroup::Base ) );
    p.end();
}

void ImageEdit::drawContents( QPainter *p, int cx, int cy, int cw, int ch )
{
    p->drawPixmap( cx, cy, buffer, cx, cy, cw, ch );
}

void ImageEdit::setPixmap( const QPixmap &pm )
{
    buffer = pm;
    viewport()->repaint( FALSE );
}

QPixmap ImageEdit::pixmap() const
{
    return buffer;
}
