/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
//
// Very simple example of QSound::play(filename)
//
// 99% of this program is just boilerplate Qt code to put up a nice
// window so you think something special is happening.
//

#include "sound.h"
#include <qmessagebox.h>
#include <qmenubar.h>
#include <stdlib.h>

SoundPlayer::SoundPlayer( QWidget *parent, const char *name, WFlags f ) :
    QMainWindow(parent,name,f)
#ifndef QT_NO_SOUND
    ,
    bucket3(new QSound(QString(getenv("QTDIR"))+"/etc/sounds/3.wav")),
    bucket4(new QSound(QString(getenv("QTDIR"))+"/etc/sounds/4.wav"))
#endif
{
#ifndef QT_NO_SOUND
    if (!QSound::available()) {
	// Bail out.  Programs in which sound is not critical
	// could just silently (hehe) ignore the lack of a server.
	//
	QMessageBox::warning(this,"No Sound",
		"<p><b>Sorry, you are not running the Network Audio System.</b>"
		"<p>If you have the `au' command, run it in the background before this program. "
		"The latest release of the Network Audio System can be obtained from:"
		"<pre>\n"
		" &nbsp;\n"
		"   ftp.ncd.com:/pub/ncd/technology/src/nas\n"
		"   ftp.x.org:/contrib/audio/nas\n"
		"</pre>"
		"<p>Release 1.2 of NAS is also included with the X11R6"
		"contrib distribution."
		"<p>After installing NAS, you will then need to reconfigure Qt with NAS sound support");
    }

    QPopupMenu *file = new QPopupMenu;
    file->insertItem("Play &1",  this, SLOT(doPlay1()), CTRL+Key_1);
    file->insertItem("Play &2",  this, SLOT(doPlay2()), CTRL+Key_2);
    file->insertItem("Play from bucket &3",  this, SLOT(doPlay3()), CTRL+Key_3);
    file->insertItem("Play from bucket &4",  this, SLOT(doPlay4()), CTRL+Key_4);
    file->insertSeparator();
    file->insertItem("Play 3 and 4 together",  this, SLOT(doPlay34()));
    file->insertItem("Play all together",  this, SLOT(doPlay1234()));
    file->insertSeparator();
    file->insertItem("E&xit",  qApp, SLOT(quit()));
    menuBar()->insertItem("&File", file);
#endif
}

void SoundPlayer::doPlay1()
{
#ifndef QT_NO_SOUND
    QSound::play(QString(getenv("QTDIR"))+"/etc/sounds/1.wav");
#endif
}

void SoundPlayer::doPlay2()
{
#ifndef QT_NO_SOUND
    QSound::play(QString(getenv("QTDIR"))+"/etc/sounds/2.wav");
#endif
}

void SoundPlayer::doPlay3()
{
#ifndef QT_NO_SOUND
    bucket3->play();
#endif
}

void SoundPlayer::doPlay4()
{
#ifndef QT_NO_SOUND
    bucket4->play();
#endif
}

void SoundPlayer::doPlay34()
{
#ifndef QT_NO_SOUND
    // Some sound platforms will only play one sound at a time
    bucket3->play();
    bucket4->play();
#endif
}

void SoundPlayer::doPlay1234()
{
#ifndef QT_NO_SOUND
    // Some sound platforms will only play one sound at a time
    QSound::play(QString(getenv("QTDIR"))+"/etc/sounds/1.wav");
    QSound::play(QString(getenv("QTDIR"))+"/etc/sounds/2.wav");
    bucket3->play();
    bucket4->play();
#endif
}

