/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <qpeapplication.h>
#include <qfile.h>
#include <qwindowsystem_qws.h>
#include "desktop.h"
#include "calibrate.h"

#ifdef QT_QWS_CASSIOPEIA
#include <sys/mount.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

void initCassiopeia()
{
    // MIPSEL-specific init - make sure /proc exists for shm
    if ( mount("/dev/ram0", "/", "ext2", MS_REMOUNT | MS_MGC_VAL, 0 ) ) {
	perror("Remounting - / read/write");
    }
    if ( mount("none","/proc","proc",0,0) ) {
	perror("Mounting - /proc");
    } else {
	fprintf( stderr, "mounted /proc\n" );
    }
    if ( mount("none","/mnt","shm",0,0) ) {
	perror("Mounting - shm");
    }
    setenv( "QTDIR", "/", 1 );
    setenv( "QPEDIR", "/usr/share", 1 );
}
#endif

int initApplication( int argc, char ** argv )
{
#ifdef QT_QWS_CASSIOPEIA
    initCassiopeia();
#endif

#if !defined(QT_QWS_CASSIOPEIA) && !defined(QT_QWS_IPAQ)
    setenv( "QWS_SIZE", "240x320", 0 );
#endif
    
    //Don't flicker at startup:
    QWSServer::setDesktopBackground( QImage() );
    QPEApplication a( argc, argv, QApplication::GuiServer );

    Desktop *d = new Desktop();
    d->show();

#if defined(QT_QWS_CASSIOPEIA) || defined(QT_QWS_IPAQ)
    if ( !QFile::exists( "/etc/pointercal" ) ) {
        Calibrate *c = new Calibrate;
        c->show();
    }
#endif

    return a.exec();
}

int main( int argc, char ** argv )
{
    int retVal = initApplication( argc, argv );
    ::killpg( ::getpid(), SIGKILL );
    return retVal;
}

