/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include <qframe.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qpainter.h>
#include <pixmaploader.h>
#include "startmenu.h"
#include "mrulist.h"


MRUList::MRUList( QWidget *parent, StartMenu *startMenu )
	: QFrame( parent ), numberOfIcons(0), selected(-1), oldsel(-1), menu(startMenu)
{
}


QSize MRUList::sizeHint() const
{
    return QSize( frameWidth(), 16 );
}


void MRUList::addTask( AppLnk *appLnk )
{
    bool alreadyInList = FALSE;
    unsigned int i = 0;

    for ( ; i < numberOfIcons; i++ )
	if ( mruAppList[i] == appLnk ) {
	    alreadyInList = TRUE;
	    if (i == 0)
		return;
	    break;
	}

    if (!alreadyInList) {
	numberOfIcons++;
	if (numberOfIcons >= ARRAY_SIZE(mruAppList))
	    numberOfIcons = ARRAY_SIZE(mruAppList) - 1;
	i = numberOfIcons;
    }
    
    for ( ; i > 0; i--)
	mruAppList[i] = mruAppList[i - 1];
    
    mruAppList[0] = appLnk;

    repaint();
}


void MRUList::mousePressEvent(QMouseEvent *e)
{
    selected = e->pos().x() / 15;
    if ( selected >= (int)numberOfIcons )
	selected = -1;
    else
	repaint( FALSE );
}


void MRUList::mouseMoveEvent(QMouseEvent *e)
{
    int i = e->pos().x() / 15;
    if ( i >= (int)numberOfIcons || !rect().contains( e->pos() ) )
	i = -1;
    if ( selected != i ) {
	oldsel = selected;
	selected = i;
	repaint( FALSE );
    }
}


void MRUList::mouseReleaseEvent(QMouseEvent *e)
{
    int i = e->pos().x() / 15;
    if (selected == i ) {
	menu->launchMenu->hide();
	mruAppList[i]->execute();
    }
    selected = -1;
    oldsel = -1;
}


void MRUList::paintEvent( QPaintEvent *event )
{
    QPainter p( this );
    int x = 0;
    
    if ( event->rect().intersects( frameRect() ) ) {
	for (unsigned char i = 0; i < numberOfIcons; i++) {
	    QString icon = mruAppList[i]->getIcon();
	    QPixmap pm( PixmapLoader::loadPixmap( icon ) );
	    if ( (int)i == selected )
		p.fillRect( x, 0, 15, pm.height()+1, colorGroup().highlight() );
	    else if ( (int)i == oldsel )
		p.fillRect( x, 0, 15, pm.height()+1, colorGroup().background() );
	    p.drawPixmap( x, 1, pm );
	    x += 15;
	}
    }
}

