/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "textedit.h"
#include "qspellchecker.h"
#include "fileselector.h"

#include <qwidgetstack.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qapplication.h>
#include <qlineedit.h>
#include <qtoolbutton.h>
#include <qcolordialog.h>
#include <qspinbox.h>
#include <qobjectlist.h>
#include <global.h>
#include <qfileinfo.h>

#include "pixmaploader.h"

static int u_id = 1;
static int get_unique_id()
{
    return u_id++;
}


#ifdef USE_MULTILINEEDIT
# define EDITOR_TYPE QMultiLineEdit
# define EDITOR_STRING "QMultiLineEdit"
#else
//# include "inserttable.h" -- WWA: don't let tmake see this
# define EDITOR_TYPE QTextEdit
# define EDITOR_STRING "QTextEdit"
#endif

TextEdit::TextEdit( QWidget *parent, const char *name, WFlags f )
    : QMainWindow( parent, name, f )
{
    setToolBarsMovable( FALSE );

    QToolBar *bar = new QToolBar( this );
    bar->setHorizontalStretchable( TRUE );
    menu = bar;

    QMenuBar *mb = new QMenuBar( bar );

    QPopupMenu *file = new QPopupMenu( this );
    QPopupMenu *edit = new QPopupMenu( this );

    bar = new QToolBar( this );
    editBar = bar;

    QAction *a = new QAction( tr( "New" ), PixmapLoader::loadPixmap( "new" ), QString::null, CTRL + Key_N, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileNew() ) );
    a->addTo( file );

    a = new QAction( tr( "Open" ), PixmapLoader::loadPixmap( "open" ), QString::null, CTRL + Key_O, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    a->addTo( file );

    a = new QAction( tr( "Close" ), PixmapLoader::loadPixmap( "close" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( fileClose() ) );
    a->addTo( file );


    a = new QAction( tr( "Cut" ), PixmapLoader::loadPixmap( "cut" ), QString::null, CTRL + Key_X, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( editCut() ) );
    a->addTo( bar );
    a->addTo( edit );

    a = new QAction( tr( "Copy" ), PixmapLoader::loadPixmap( "copy" ), QString::null, CTRL + Key_C, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( editCopy() ) );
    a->addTo( bar );
    a->addTo( edit );

    a = new QAction( tr( "Paste" ), PixmapLoader::loadPixmap( "paste" ), QString::null, CTRL + Key_C, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( editPaste() ) );
    a->addTo( bar );
    a->addTo( edit );

    a = new QAction( tr( "Find..." ), PixmapLoader::loadPixmap( "find" ), QString::null, CTRL + Key_F, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( editFind() ) );
    edit->insertSeparator();
    a->addTo( edit );

    a = new QAction( tr( "Find Next" ), PixmapLoader::loadPixmap( "next" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( findNext() ) );
    a->addTo( bar );
    a->addTo( edit );

    QPopupMenu *insertMenu = new QPopupMenu( this );
    edit->insertSeparator();
    edit->insertItem( tr( "&Insert" ), insertMenu );

    a = new QAction( tr( "Table..." ), QPixmap(), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( insertTable() ) );
    a->addTo( insertMenu );

    mb->insertItem( tr( "&File" ), file );
    mb->insertItem( tr( "&Edit" ), edit );

    formatBar = new QToolBar( "Format", this, QMainWindow::Top, TRUE );
    searchBar = new QToolBar( "Search", this, QMainWindow::Top, TRUE );

    a = new QAction( tr( "Bold" ), PixmapLoader::loadPixmap( "bold" ), QString::null, CTRL + Key_B, this, 0 );
    connect( a, SIGNAL( toggled( bool ) ), this, SLOT( formatBold( bool ) ) );
    a->setToggleAction( TRUE );
    a->addTo( formatBar );
    fBold = a;
    a = new QAction( tr( "Italic" ), PixmapLoader::loadPixmap( "italic" ), QString::null, CTRL + Key_I, this, 0 );
    connect( a, SIGNAL( toggled( bool ) ), this, SLOT( formatItalic( bool ) ) );
    a->setToggleAction( TRUE );
    a->addTo( formatBar );
    fItalic = a;
    a = new QAction( tr( "Underline" ), PixmapLoader::loadPixmap( "underline" ), QString::null, CTRL + Key_U, this, 0 );
    connect( a, SIGNAL( toggled( bool ) ), this, SLOT( formatUnderline( bool ) ) );
    a->addTo( formatBar );
    a->setToggleAction( TRUE );
    fUnderline = a;

    QActionGroup *g = new QActionGroup( this );
    g->setExclusive( TRUE );

    a = new QAction( tr( "Left" ), PixmapLoader::loadPixmap( "left" ), QString::null, CTRL + Key_L, g, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( formatLeft() ) );
    a->addTo( formatBar );
    a->setToggleAction( TRUE );
    a->setOn( TRUE );
    alignLeft = a;
    a = new QAction( tr( "Center" ), PixmapLoader::loadPixmap( "center" ), QString::null, CTRL + Key_C, g, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( formatCenter() ) );
    a->addTo( formatBar );
    a->setToggleAction( TRUE );
    alignCenter = a;
    a = new QAction( tr( "Right" ), PixmapLoader::loadPixmap( "right" ), QString::null, CTRL + Key_R, g, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( formatRight() ) );
    a->addTo( formatBar );
    a->setToggleAction( TRUE );
    alignRight = a;

    QToolButton *b = new QToolButton( formatBar );
    b->setPixmap( PixmapLoader::loadPixmap( "style" ) );

    styleMenu = new QPopupMenu( b );
    b->setPopup( styleMenu );
    b->setPopupDelay( 0 );

    colorButton = new QToolButton( formatBar );
    colorChanged( black );
    connect( colorButton, SIGNAL( clicked() ),
	     this, SLOT( colorClicked() ) );

    searchBar->setHorizontalStretchable( TRUE );

    searchEdit = new QLineEdit( searchBar, "searchEdit" );
    searchBar->setStretchableWidget( searchEdit );
    connect( searchEdit, SIGNAL( textChanged( const QString & ) ),
	     this, SLOT( search() ) );

    a = new QAction( tr( "Close Find" ), PixmapLoader::loadPixmap( "close" ), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( findClose() ) );
    a->addTo( searchBar );

    searchBar->hide();
		
    editorStack = new QWidgetStack( this );
    setCentralWidget( editorStack );

    searchVisible = FALSE;

    fileSelector = new FileSelector( "*.html;*.htm", editorStack, "fileselector" );
    connect( fileSelector, SIGNAL( closeMe() ), this, SLOT( closeFileSelector() ) );
    connect( fileSelector, SIGNAL( newSelected( const FileManager::File &) ), this, SLOT( newFile( const FileManager::File & ) ) );
    connect( fileSelector, SIGNAL( fileSelected( const FileManager::File &) ), this, SLOT( openFile( const FileManager::File & ) ) );
    lastEditor = 0;
    fileOpen();

    a = new QAction( tr( "Standard" ), QPixmap(), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( styleStandard() ) );
    a->addTo( styleMenu );
    a = new QAction( tr( "Bullet List" ), QPixmap(), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( styleBulletList() ) );
    a->addTo( styleMenu );
    a = new QAction( tr( "Enumerated List" ), QPixmap(), QString::null, 0, this, 0 );
    connect( a, SIGNAL( activated() ), this, SLOT( styleEnumeratedList() ) );
    a->addTo( styleMenu );

#ifdef USE_MULTILINEEDIT
    formatBar->hide();
#endif

    resize( 200, 300 );
}

TextEdit::~TextEdit()
{
    save();
}

void TextEdit::fileNew()
{
    save();
    clear();

    EDITOR_TYPE *e = new EDITOR_TYPE( editorStack );

    editorStack->addWidget( e, get_unique_id() );
    editorList.append( e );
    editorStack->raiseWidget( e );
    lastEditor = editorStack->visibleWidget();

#ifndef USE_MULTILINEEDIT    
    e->viewport()->setFocus();
    e->document()->setSyntaxHighlighter( new QSpellChecker( e->document() ) );

    connect( e, SIGNAL( currentFontChanged( const QFont & ) ), this, SLOT( textFontChanged( const QFont & ) ) );
    connect( e, SIGNAL( currentColorChanged( const QColor & ) ), this, SLOT( textColorChanged( const QColor & ) ) );
    connect( e, SIGNAL( currentAlignmentChanged( int ) ), this, SLOT( textAlignmentChanged( int ) ) );
#endif
    
    FileManager::File f;
    f.filename = "Unnamed";
    f.category = Global::categories().first();

    fileMap.insert( (EDITOR_TYPE*)editorStack->visibleWidget(), f );

    updateCaption();
}

void TextEdit::fileOpen()
{
    save();
    menu->hide();
    editBar->hide();
    searchBar->hide();
    formatBar->hide();
    lastEditor = editorStack->visibleWidget();
    editorStack->raiseWidget( fileSelector );
    fileSelector->setCloseVisible( !(bool)lastEditor );
    fileSelector->setCloseVisible( (bool)lastEditor );
    fileSelector->reread();
    updateCaption();
}

void TextEdit::fileClose()
{
    clear();
    fileOpen();
}

void TextEdit::editCut()
{
#ifndef QT_NO_CLIPBOARD
    currentEditor()->cut();
#endif
}

void TextEdit::editCopy()
{
#ifndef QT_NO_CLIPBOARD
    currentEditor()->copy();
#endif
}

void TextEdit::editPaste()
{
#ifndef QT_NO_CLIPBOARD
    currentEditor()->paste();
#endif
}

void TextEdit::editFind()
{
    formatBar->hide();
    searchBar->show();
    searchVisible = TRUE;
    searchEdit->setFocus();
}

void TextEdit::formatBold( bool b )
{
#ifndef USE_MULTILINEEDIT
    currentEditor()->blockSignals( TRUE );
    currentEditor()->setBold( b );
    currentEditor()->blockSignals( FALSE );
#endif
}

void TextEdit::formatItalic( bool b )
{
#ifndef USE_MULTILINEEDIT
    currentEditor()->blockSignals( TRUE );
    currentEditor()->setItalic( b );
    currentEditor()->blockSignals( FALSE );
#endif
}

void TextEdit::formatUnderline( bool b )
{
#ifndef USE_MULTILINEEDIT
    currentEditor()->blockSignals( TRUE );
    currentEditor()->setUnderline( b );
    currentEditor()->blockSignals( FALSE );
#endif
}

void TextEdit::formatLeft()
{

    currentEditor()->blockSignals( TRUE );
    currentEditor()->setAlignment( AlignLeft );
    currentEditor()->blockSignals( FALSE );

}

void TextEdit::formatCenter()
{
    currentEditor()->blockSignals( TRUE );
    currentEditor()->setAlignment( AlignCenter );
    currentEditor()->blockSignals( FALSE );
}

void TextEdit::formatRight()
{
    currentEditor()->blockSignals( TRUE );
    currentEditor()->setAlignment( AlignRight );
    currentEditor()->blockSignals( FALSE );
}

void TextEdit::findNext()
{
#ifndef USE_MULTILINEEDIT
    QString s = searchEdit->text();
    currentEditor()->find( s, FALSE, FALSE );
#endif
}

void TextEdit::findClose()
{
    searchVisible = FALSE;
    searchBar->hide();
#ifndef USE_MULTILINEEDIT
    formatBar->show();
    currentEditor()->viewport()->setFocus();
#endif
}

void TextEdit::styleStandard()
{
#ifndef USE_MULTILINEEDIT
    currentEditor()->setParagType( QStyleSheetItem::DisplayBlock, -1 );
#endif
}

void TextEdit::styleBulletList()
{
#ifndef USE_MULTILINEEDIT
    currentEditor()->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListDisc );
#endif
}

void TextEdit::styleEnumeratedList()
{
#ifndef USE_MULTILINEEDIT
    currentEditor()->setParagType( QStyleSheetItem::DisplayListItem, QStyleSheetItem::ListDecimal );
#endif
}

void TextEdit::colorClicked()
{
#ifndef USE_MULTILINEEDIT
    QColor col = QColorDialog::getColor( QColor(), this );
    if ( col.isValid() ) {
	currentEditor()->blockSignals( TRUE );
	currentEditor()->setColor( col );
	currentEditor()->blockSignals( FALSE );
	colorChanged( col );
    }
#endif
}

void TextEdit::search()
{
    QString s = searchEdit->text();
    int i = 0;
#ifndef USE_MULTILINEEDIT
    currentEditor()->find( s, FALSE, FALSE, TRUE, &i, &i );
#endif
}


EDITOR_TYPE *TextEdit::currentEditor() const
{
    if ( editorStack->visibleWidget() && editorStack->visibleWidget()->inherits( EDITOR_STRING ) )
	return (EDITOR_TYPE*)editorStack->visibleWidget();
    return 0;
}

void TextEdit::colorChanged( const QColor &c )
{
    QPixmap pix( 16, 16 );
    pix.fill( c );
    colorButton->setPixmap( pix );
}

void TextEdit::textColorChanged( const QColor &c )
{
    colorChanged( c );
}

void TextEdit::textAlignmentChanged( int a )
{
    if ( ( a & AlignCenter ) == AlignCenter )
	alignCenter->setOn( TRUE );
    else if ( ( a & AlignLeft ) == AlignLeft )
	alignLeft->setOn( TRUE );
    else if ( ( a & AlignRight ) == AlignRight )
	alignRight->setOn( TRUE );
}

void TextEdit::textFontChanged( const QFont &f )
{
    fBold->setOn( f.bold() );
    fItalic->setOn( f.italic() );
    fUnderline->setOn( f.underline() );
}

void TextEdit::insertTable()
{
#ifndef USE_MULTILINEEDIT
    InsertTable t( this, 0, TRUE );
    t.exec();
    int rows = t.spinRows->value();
    int cols = t.spinColumns->value();

    QMap<QString,QString> attr;
    attr.insert( "border", "1" );
    QTextTable* table = new QTextTable( currentEditor()->document(), attr );
    for ( int r = 0; r < rows; ++r ) {
	for ( int c = 0; c < cols; ++c )
	    (void)new QTextTableCell( table, r, c );
    }

    currentEditor()->insert( "\n ", FALSE, TRUE );
    currentEditor()->textCursor()->parag()->at( 0 )->setCustomItem( table );
    currentEditor()->document()->registerCustomItem( table, currentEditor()->textCursor()->parag() );
    currentEditor()->document()->invalidate();
    currentEditor()->viewport()->repaint( FALSE );
#endif
}

void TextEdit::newFile( const FileManager::File &f )
{
    clear();
    fileNew();
    fileMap.remove( (EDITOR_TYPE*)editorStack->visibleWidget() );
    fileMap.insert( (EDITOR_TYPE*)editorStack->visibleWidget(), f );
    updateCaption();
}

void TextEdit::openFile( const FileManager::File &f )
{
    clear();
    FileManager fm;
    QByteArray ba;
    if ( !fm.loadFile( f, ba ) ) {
	qDebug( "Cannot open file" );
	return;
    }
    QTextStream ts( ba, IO_ReadOnly );
    QString txt = ts.read();
    fileNew();
    fileMap.insert( (EDITOR_TYPE*)editorStack->visibleWidget(), f );
#ifndef USE_MULTILINEEDIT
    currentEditor()->document()->setRichText( txt, QString::null );
    currentEditor()->setModified(FALSE);
#else
    currentEditor()->setText( txt );
    currentEditor()->setEdited(FALSE);
#endif
    updateCaption();
}

void TextEdit::closeFileSelector()
{
    if ( lastEditor ) {
	editorStack->raiseWidget( lastEditor );
	menu->show();
	editBar->show();
	if ( searchVisible )
	    searchBar->show();
#ifndef USE_MULTILINEEDIT
	else
	    formatBar->show();
#endif
    } else if ( currentEditor() )
	close();
    updateCaption();
}

void TextEdit::save()
{
    if ( !currentEditor() )
	return;
#ifdef USE_MULTILINEEDIT
    if ( !currentEditor()->edited() )
#else
    if ( !currentEditor()->isModified() )
#endif
	return;
    QByteArray ba;
    QTextStream ts( ba, IO_WriteOnly );
#ifdef USE_MULTILINEEDIT
    QString rt = currentEditor()->text();
    QString pt = rt;
#else
    QString rt = currentEditor()->document()->richText();
    QString pt = currentEditor()->document()->plainText();
#endif
    ts << rt;
    FileManager::File f = fileMap[ currentEditor() ];
    if ( f.filename == "Unnamed" ) {
	int i = pt.find( ' ' );
	if ( i == -1 ) {
	    if ( pt.isEmpty() )
		f.filename = "Empty Text";
	    else
		f.filename = pt;
	} else {
	    f.filename = pt.left( i );
	}
    }
    QFileInfo fi( f.filename );
    if ( fi.extension() != "html" )
	f.filename += ".html";
    FileManager fm;
    fm.saveFile( f, ba );
}

void TextEdit::clear()
{
    QObjectListIt it( *editorStack->children() );
    QObject *o;
    while ( ( o = it.current() ) ) {
	++it;
	if ( o->inherits( EDITOR_STRING ) )
	    delete o;
    }
    lastEditor = 0;
}

void TextEdit::updateCaption()
{
    if ( !currentEditor() )
	setCaption( tr("Text Editor") );
    else {
	FileManager::File f = fileMap[ currentEditor() ];
	setCaption( f.filename + " - " + tr("Text Editor") );
    }
}

