/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef TEXTEDIT_H
#define TEXTEDIT_H

#define QTEXTEDIT_OPEN_API


#include <qmainwindow.h>
#ifdef USE_MULTILINEEDIT
# include <qmultilineedit.h>
#else
# include "qtextedit.h"
#endif
#include <qlist.h>
#include <filemanager.h>
#include <qmap.h>

class QWidgetStack;
class QPopupMenu;
class QToolBar;
class QLineEdit;
class QAction;
class FileSelector;

class TextEdit : public QMainWindow
{
    Q_OBJECT

public:
    TextEdit( QWidget *parent = 0, const char *name = 0, WFlags f = 0 );
    ~TextEdit();
    
private slots:
    void fileNew();
    void fileClose();
    void fileOpen();

    void editCut();
    void editCopy();
    void editPaste();
    void editFind();

    void formatBold( bool );
    void formatItalic( bool );
    void formatUnderline( bool );
    void formatLeft();
    void formatCenter();
    void formatRight();

    void findNext();
    void findClose();

    void styleStandard();
    void styleBulletList();
    void styleEnumeratedList();

    void colorClicked();

    void search();

    void textColorChanged( const QColor &c );
    void textAlignmentChanged( int a );
    void textFontChanged( const QFont &f );

    void insertTable();
    void newFile( const FileManager::File & );
    void openFile( const FileManager::File & );
    void closeFileSelector();

private:
#ifdef USE_MULTILINEEDIT
    QMultiLineEdit *currentEditor() const;
#else    
    QTextEdit *currentEditor() const;
#endif
    void colorChanged( const QColor &c );
    void save();
    void clear();
    void updateCaption();
    
private:
    QWidgetStack *editorStack;
    QWidget *lastEditor;
    FileSelector *fileSelector;
#ifdef USE_MULTILINEEDIT
    QList<QMultiLineEdit> editorList;
#else    
    QList<QTextEdit> editorList;
#endif
    QPopupMenu *styleMenu;
    QToolBar *menu, *editBar, *formatBar, *searchBar;
    QLineEdit *searchEdit;
    QToolButton *colorButton;
    QAction *alignLeft, *alignCenter, *alignRight;
    QAction *fBold, *fItalic, *fUnderline;
    bool searchVisible;
#ifdef USE_MULTILINEEDIT
    QMap<QMultiLineEdit*, FileManager::File> fileMap;
#else
    QMap<QTextEdit*, FileManager::File> fileMap;
#endif    
};

#endif
