/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "mainwindow.h"
#include "todotable.h"
#include "todoentryimpl.h"

#include <organizer.h>
#include <pixmaploader.h>

#include <qtoolbar.h>
#include <qaction.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qapplication.h>

#include <stdlib.h>

TodoWindow::TodoWindow( QWidget *parent, const char *name, WFlags f = 0 ) :
    QMainWindow( parent, name, f ),
    filename( QString( getenv( "HOME" ) ) + "/.todo" )
{
    setCaption( tr("Todo") );
    table = new TodoTable( this );
    connect( table, SIGNAL( currentChanged( int, int ) ),
	     this, SLOT( currentEntryChanged( int, int ) ) );
    setCentralWidget( table );
    setToolBarsMovable( FALSE );

    QToolBar *bar = new QToolBar( this );
    bar->setHorizontalStretchable( TRUE );

    QMenuBar *mb = new QMenuBar( bar );

    QPopupMenu *file = new QPopupMenu( this );
    QPopupMenu *edit = new QPopupMenu( this );

    bar = new QToolBar( this );

    QAction *a = new QAction( tr( "New Task" ), PixmapLoader::loadPixmap( "new" ), QString::null, CTRL + Key_N, this, 0 );
    connect( a, SIGNAL( activated() ),
	     this, SLOT( slotNew() ) );
    a->addTo( bar );
    a->addTo( file );

    file->insertItem( tr( "Close" ), this, SLOT( close() ) );

    a = new QAction( tr( "Edit" ), PixmapLoader::loadPixmap( "edit" ), QString::null, CTRL + Key_E, this, 0 );
    connect( a, SIGNAL( activated() ),
	     this, SLOT( slotEdit() ) );
    a->addTo( bar );
    a->addTo( edit );
    a->setEnabled( FALSE );
    editAction = a;
    
    a = new QAction( tr( "Delete" ), PixmapLoader::loadPixmap( "delete" ), QString::null, CTRL + Key_Delete, this, 0 );
    connect( a, SIGNAL( activated() ),
	     this, SLOT( slotDelete() ) );
    a->addTo( bar );
    a->addTo( edit );
    a->setEnabled( FALSE );
    deleteAction = a;
    
    mb->insertItem( tr( "File" ), file );
    mb->insertItem( tr( "Task" ), edit );

    resize( 200, 300 );
    table->setColumnWidth( 2, 10 );
    table->load( filename );
    if ( table->numRows() > 0 )
	currentEntryChanged( 0, 0 );
}

void TodoWindow::slotNew()
{
    NewTaskDialog e( this, 0, TRUE );

#ifdef _WS_QWS_
    e.showMaximized();
#endif
#if defined(QT_HAS_OK_IN_TITLEBAR)
    if ( e.exec() != QDialog::Accepted )
	return;
#else
    e.exec();
#endif

    Todo *todo = e.todoEntry();
#if !defined(QT_HAS_OK_IN_TITLEBAR)
    if ( todo->description().isEmpty() ||
	 todo->description() == tr( "<Nothing>" ) ) {
	QMessageBox::information( this, tr( "Todo" ), tr( "You haven't entered a description!" ) );
	return;
    }
#endif

    table->addEntry( todo );
    table->save( filename );

    currentEntryChanged( 0, 0 );
}

void TodoWindow::slotDelete()
{
    if ( table->currentRow() == -1 )
	return;
    table->clearCell( table->currentRow(), 0 );
    table->save( filename );
    table->clear();
    table->load( filename );
    table->save( filename );

    if ( table->numRows() == 0 )
	currentEntryChanged( -1, 0 );
}

void TodoWindow::slotEdit()
{
    Todo *todo = table->currentEntry();
    if ( !todo )
	return;

    NewTaskDialog e( this, 0, TRUE );
    e.setCaption( tr( "Edit Task" ) );
    e.init( todo );

#ifdef _WS_QWS_
    e.showMaximized();
#endif
#if defined(QT_HAS_OK_IN_TITLEBAR)
    if ( e.exec() != QDialog::Accepted )
	return;
#else
    e.exec();
#endif

    todo = e.todoEntry();
    if ( todo->description().isEmpty() ||
	 todo->description() == tr( "<Nothing>" ) ) {
	QMessageBox::information( this, tr( "Todo" ), tr( "You haven't entered a description!" ) );
	return;
    }

    table->replaceEntry( todo );
    table->save( filename );
}


void TodoWindow::currentEntryChanged( int r, int )
{
    if ( r != -1 ) {
	editAction->setEnabled( TRUE );
	deleteAction->setEnabled( TRUE );
    } else {
	editAction->setEnabled( FALSE );
	deleteAction->setEnabled( FALSE );
    }
}
