/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qt Palmtop Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#ifndef TODOTABLE_H
#define TODOTABLE_H

#include <qtable.h>
#include <qmap.h>

#include <organizer.h>
#include <qguardedptr.h>

class Node;
class QComboBox;

class CheckItem : public QTableItem
{
public:
    CheckItem( QTable *t );

    void setChecked( bool b );
    void toggle();
    bool isChecked() const;

    void paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bool selected );

private:
    bool checked;

};

class ComboItem : public QTableItem
{
public:
    ComboItem( QTable *t, EditType et );
    QWidget *createEditor() const;
    void setContentFromEditor( QWidget *w );
    void setText( const QString &s );
    int alignment() const { return Qt::AlignCenter; }

    QString text() const;

private:
    QGuardedPtr<QComboBox> cb;

};

class TodoTable : public QTable
{
    Q_OBJECT

public:
    TodoTable( QWidget *parent = 0, const char * name = 0 );
    void addEntry( Todo *todo );

    Todo *currentEntry() const;
    void replaceEntry( Todo *todo );

    void save( const QString &fn );
    void load( const QString &fn );
    void clear();
    
private slots:
    void slotClicked( int row, int col, int button, const QPoint &pos );

private:
    QMap<CheckItem*, Todo*> todoList;

};

#endif
